from .use_cases import Formatter
from .core import Codes


def output(function):
    def wrapper(*args, **kwargs):
        return_value = function(*args, **kwargs)

        print(return_value)
        if kwargs.get('flush'):
            print(Codes.line_up, end=Codes.line_clear)

    return wrapper


class Logger:
    '''
    The Logger class can be used in two ways: it can be instanced with a name or used as a static class for quick outputs.
    
    The Logger class has four logging levels: info, debug, warning and error.
    '''
    name: str = ''
    info_prefix: str = Formatter.green(Formatter.bold('[INFO]'))
    debug_prefix: str = Formatter.blue(Formatter.bold('[DEBUG]'))
    warning_prefix: str = Formatter.yellow(Formatter.bold('[WARNING]'))
    error_prefix: str = Formatter.red(Formatter.bold('[ERROR]'))

    
    def __init__(self, name: str) -> None:
        '''
        Arguments
        ---------
        name: str
            the name of the instance
        '''
        __class__.name = name

    @output
    @staticmethod
    def info(*message, sep: str = '', flush: bool = False) -> str:
        '''        
        Log the given message at the info level.
        
        Arguments
        ---------
        *message: Any | *tuple[Any]
            the message to log
        
        sep: str, optional
            the separation of the given inputs. Default to ''
        
        flush: bool, optional
            when set to True, the inputs will stay in the same line. Default to False
        '''        
        if isinstance(message[0], Logger):
            return __class__.info_prefix + Formatter.green(f'    [{__class__.name}] {sep.join(map(str, message[1:]))}')
        else:
            return __class__.info_prefix + Formatter.green(f'    {sep.join(map(str, message))}')

    @output
    @staticmethod
    def debug(*message, sep: str = '', flush: bool = False) -> str:
        '''
        Log the given message at the debug level.
        
        Arguments
        ---------
        *message: Any | *tuple[Any]
            the message to log
        
        sep: str, optional
            the separation of the given inputs. Default to ''
        
        flush: bool, optional
            when set to True, the inputs will stay in the same line. Default to False
        '''        
        if isinstance(message[0], Logger):
            return __class__.debug_prefix + Formatter.blue(f'   [{__class__.name}] {sep.join(map(str, message[1:]))}')
        else:
            return __class__.debug_prefix + Formatter.blue(f'   {sep.join(map(str, message))}')


    @output
    @staticmethod
    def warning(*message, sep: str = '', flush: bool = False) -> str:
        '''
        Log the given message at the warning level.
        
        Arguments
        ---------
        *message: Any | *tuple[Any]
            the message to log
        
        sep: str, optional
            the separation of the given inputs. Default to ''
        
        flush: bool, optional
            when set to True, the inputs will stay in the same line. Default to False
        '''        
        if isinstance(message[0], Logger):
            return __class__.warning_prefix + Formatter.yellow(f' [{__class__.name}] {sep.join(map(str, message[1:]))}')
        else:
            return __class__.warning_prefix + Formatter.yellow(f' {sep.join(map(str, message))}')


    @output
    @staticmethod
    def error(*message, sep: str = '', flush: bool = False) -> str:
        '''
        Log the given message at the error level.
        
        Arguments
        ---------
        *message: Any | *tuple[Any]
            the message to log
        
        sep: str, optional
            the separation of the given inputs. Default to ''
        
        flush: bool, optional
            when set to True, the inputs will stay in the same line. Default to False
        '''
        if isinstance(message[0], Logger):
            return __class__.error_prefix + Formatter.red(f'   [{__class__.name}] {sep.join(map(str, message[1:]))}')
        else:
            return __class__.error_prefix + Formatter.red(f'   {sep.join(map(str, message))}')




import dataclasses


@dataclasses.dataclass
class Codes:
    end = "\x1b[0m"
    bold = "\x1b[1m"
    dim = "\x1b[2m"
    italic = "\x1b[2m"
    underscore = "\x1b[4m"
    blink = "\x1b[5m"
    highlight = "\x1b[7m"
    hidden = "\x1b[8m"
    strikethrough = "\x1b[9m"
    double_underscore = "\x1b[21m"

    black = "\x1b[30m"
    red = "\x1b[31m"
    green = "\x1b[32m"
    yellow = "\x1b[33m"
    blue = "\x1b[34m"
    magenta = "\x1b[35m"
    cyan = "\x1b[36m"
    white = "\x1b[37m"

    black_bg = "\x1b[40m"
    red_bg = "\x1b[41m"
    green_bg = "\x1b[42m"
    yellow_bg = "\x1b[43m"
    blue_bg = "\x1b[44m"
    magenta_bg = "\x1b[45m"
    cyan_bg = "\x1b[46m"
    white_bg = "\x1b[47m"

    secondary_black = "\x1b[90m"
    secondary_red = "\x1b[91m"
    secondary_green = "\x1b[92m"
    secondary_yellow = "\x1b[93m"
    secondary_blue = "\x1b[94m"
    secondary_magenta = "\x1b[95m"
    secondary_cyan = "\x1b[96m"
    secondary_white = "\x1b[97m"

    secondary_black_bg = "\x1b[40m"
    secondary_red_bg = "\x1b[41m"
    secondary_green_bg = "\x1b[42m"
    secondary_yellow_bg = "\x1b[43m"
    secondary_blue_bg = "\x1b[44m"
    secondary_magenta_bg = "\x1b[45m"
    secondary_cyan_bg = "\x1b[46m"
    secondary_white_bg = "\x1b[47m"

    line_up = "\x1b[1A"
    line_clear = "\x1b[2K"


from typing import NamedTuple


class Colors(NamedTuple):
    black = "\x1b[30m"
    red = "\x1b[31m"
    green = "\x1b[32m"
    yellow = "\x1b[33m"
    blue = "\x1b[34m"
    magenta = "\x1b[35m"
    cyan = "\x1b[36m"
    white = "\x1b[37m"
    
    secondary_black = "\x1b[90m"
    secondary_red = "\x1b[91m"
    secondary_green = "\x1b[92m"
    secondary_yellow = "\x1b[93m"
    secondary_blue = "\x1b[94m"
    secondary_magenta = "\x1b[95m"
    secondary_cyan = "\x1b[96m"
    secondary_white = "\x1b[97m"



from ..core.codes import Codes


class Formatter:
    @staticmethod
    def bold(message: str) -> str:
        return Codes.bold + message + Codes.end

    @staticmethod
    def green(message: str) -> str:
        return Codes.secondary_green + message + Codes.end

    @staticmethod
    def blue(message: str) -> str:
        return Codes.secondary_blue + message + Codes.end

    @staticmethod
    def yellow(message: str) -> str:
        return Codes.yellow + message + Codes.end

    @staticmethod
    def red(message: str) -> str:
        return Codes.red + message + Codes.end